#!/usr/bin/env python3
# coding=utf-8
# author: @netmanchris
# -*- coding: utf-8 -*-
"""
This module contains contains a single function to return the current
version of the pyhpeimc library. The function should return the
current value the library and should be updated with every
new version

"""


def version():
    """Function takes no aruguments and returns a STR value of the current
    version of the library. This value should match
    the value in the test_machine.py
    :return str value of the current version of the library
    :rtype str

    >>> version()
    1.0.44
    """
    print('1.0.44')
